from django import template


class SimpleConditionNode(template.Node):
    child_nodelists = ('nodelist_true', 'nodelist_false')

    def __init__(self, nodelist_true, nodelist_false, name):
        self.nodelist_true = nodelist_true
        self.nodelist_false = nodelist_false
        self.name = name

    def repr(self):
        return "<SimpleConditionNode %s>" % self.name

    def render(self, context):
        return self.nodelist_true.render(context) + '\n' + \
            self.nodelist_false.render(context)

    @classmethod
    def handle_token(cls, parser, token, kind):
        nodelist_true = parser.parse(('else', 'end%s' % kind))
        token = parser.next_token()
        if token.contents == 'else':
            nodelist_false = parser.parse(('end%s' % kind, ))
            parser.delete_first_token()
        else:
            nodelist_false = template.NodeList()
        return cls(nodelist_true, nodelist_false, kind)



#!/usr/bin/python
import os
import unittest

import lxml.html
from django.template import loader

from . import util
from . import parse_template


def _get_test_template_dir():
    """ returns the test template directory. """
    return os.path.join(os.path.dirname(
        os.path.abspath(__file__)), 'test_templates/')


def _get_expecting_vuln(doc, xpath=None):
    ret = []
    if xpath is None:
        xpath = ".//div[@type='vuln']"
    for elem in doc.findall(xpath):
        ret.append({
            'name': elem.attrib['name'],
            'line_number': elem.sourceline})
    return ret


class TestXSSDetection(unittest.TestCase):
    """ test xss detection in templates. """

    @classmethod
    def setUpClass(cls):
        util.configure_django([_get_test_template_dir()])

    def setUp(self):
        self.template_dir = _get_test_template_dir()
        self.csw = parse_template.CompileStringWrapper()
        util.patch(self.csw)

    def _test_template_tag(self, tag_file):
        """ test a template tag """
        return self._test_template(os.path.join("tags", tag_file))

    def test_autoescape_tag(self):
        return self._test_template_tag("autoescape.html")

    def test_extends_tag(self):
        extends_dir = 'extends'
        for _file in os.listdir(os.path.join(
                self.template_dir,
                'tags', extends_dir)):
            self.setUp()
            template_name = os.path.join(extends_dir, _file)
            self._test_template_tag(template_name)

    @unittest.skip("firstof tag detection has not yet been implemented")
    def test_firstof_tag(self):
        return self._test_template_tag("firstof.html")

    def test_for_tag(self):
        return self._test_template_tag("for.html")

    def test_ifchanged_tag(self):
        return self._test_template_tag("ifchanged.html")

    def test_ifequal_tag(self):
        return self._test_template_tag("ifequal.html")

    def test_if_tag(self):
        return self._test_template_tag("if.html")

    def test_include_tag(self):
        return self._test_template_tag("include/includer.1.html")

    def test_include_tag_with_context(self):
        """ where an include node includes a template with a 'context'
            parameter name a TypeError is raised.
        """
        with self.assertRaises(TypeError):
            self._test_template_tag("include/includer.1.with_context.html")

    def test_include_missing_include(self):
        return self._test_template_tag("include/missing.html")

    def test_with_tag(self):
        return self._test_template_tag("with.html")

    def test_custom_waffle_tag(self):
        return self._test_template_tag("custom/waffle.html")

    @unittest.expectedFailure
    def test_include_in_for_tag_tag(self):
        """ include tag in for loop is not handled correctly """
        return self._test_template_tag("include/includer.2.html")

    def test_attr_injection_variable_detection(self):
        return self._test_template("attribute/injection.html")

    def test_javascript_variable_detection(self):
        """ tests detection for variables in between <script> tags
            that do not have the 'escapejs' filter
        """
        return self._test_template("javascript/javascript.html")

    @unittest.expectedFailure
    def test_javascript_variable_detection_inside_verbatim_block(self):
        """ detection of variables in between script tags - where a
            verbatim block is opened before the script tag is not handled
            correctly.
        """
        return self._test_template("javascript/verbatim.html")

    def test_walk_templates(self):
        """ test walk_templates """
        results = util.walk_templates([self.template_dir])
        self.assertTrue(results)

    def test_uniquify_results(self):
        """ test uniquify_results """
        results = util.walk_templates([self.template_dir])
        fname_and_counts = [
            ('uniquify_results/lower.html', 2),
            ('uniquify_results/top.html', 1),
            ('uniquify_results/include.html', 1),
        ]
        for fname, count in fname_and_counts:
            self.assertEqual(len(results[fname]), count)

    def _test_template(self, template_path):
        """ test that the detector finds the problems
            in a given template file.
        """
        full_path = os.path.join(self.template_dir, template_path)
        doc = lxml.html.parse(full_path)
        expecting_vuln = _get_expecting_vuln(doc)
        templ = loader.get_template(template_path)
        context = parse_template.get_default_context()
        templ.render(context)
        methods = [
            parse_template.get_non_js_escaped_results_for_template,
            parse_template.get_non_quoted_attr_vars_for_template
        ]
        for method in methods:
            for result in method(templ):
                self.csw.handle_callback(result)
        self.assertEqual(len(self.csw.results), len(expecting_vuln))
        for result, expected in zip(self.csw.results, expecting_vuln):
            line_no = result.get_line_number()
            part = result.get_vulnerability_text()
            filename = result.get_filename()
            var = str(result._var_node.filter_expression.var)
            self.assertEqual(line_no, expected['line_number'])
            self.assertEqual(var, expected['name'])
            self.assertEqual(filename, full_path)
            self.assertTrue(var in part)


class TestUtil(unittest.TestCase):
    """ test util methods. """

    def test_get_non_quoted_content(self):
        method = util.get_non_quoted_content
        for content in ["a'b'cdc", 'a"b"cdc', """a'b'" "cdc"""]:
            self.assertEqual(method(content), "acdc")


if __name__ == "__main__":
    unittest.main()



import copy

import django
import lxml.etree
import lxml.html
from django.template.base import Context
from django.template import debug
from django.utils import six


def node_has_a_filter(node, filter_func_name):
    """ returns True if the node has the given filter_func_name
        , e.g. 'safe',  otherwise returns False.
    """
    if not hasattr(node, 'filter_expression'):
        return False

    for _filter in node.filter_expression.filters:
        if _filter and _filter[0].__name__ == filter_func_name:
            return True
    return False


class InceptionDictionary(dict):
    """ subclass dict to return a dictionary that always
        has the key requested. The purpose of this is to ensure that
        django template variable and their attribute look ups always
        can always be 'resolved'.
    """

    def __str__(self):
        if '__self__' in self:
            return self['__self__']
        return super(InceptionDictionary, self).__str__()

    def __missing__(self, key):
        """ if the key cannot be found then return a new instance
            of this class.
        """
        return self.__class__()

    def __iter__(self):
        return six.moves.zip(self.items())

    def __reversed__(self):
        return self.__iter__()


def _get_var_node_source_info(var_node):
    """ returns the line number, part and filename
        for a given variable node.
    """
    source = var_node.source[0]
    string_range = var_node.source[1]
    filename = getattr(source, 'name', None)
    line_no = None
    part = None
    if hasattr(source, 'loader'):
        origin = source.loader(source.loadname)[0]
        line_no = 1 + origin[: string_range[0]].count("\n")
        part = origin[string_range[0]: string_range[1]]
    return line_no, part, filename


class IfChangedNodeOverload(django.template.defaulttags.IfChangedNode):
    """ this is used to replace the default
        IfChangedNode class - to render all condition nodelists.
    """

    def render(self, context):
        return (self.nodelist_true.render(context) + '\n' +
                self.nodelist_false.render(context))


class IfNodeOverload(django.template.defaulttags.IfNode):
    """ this is used to replace the default
        IfNode class - to render all condition nodelists
    """

    def render(self, context):
        ret = []
        for condition, nodelist in self.conditions_nodelists:
            ret.append(nodelist.render(context))
        return '\n'.join(ret)


class IfEqualNodeOverload(django.template.defaulttags.IfEqualNode):
    """ this is used to replace the default
        IfEqualNode class - to render all condition nodelists
    """

    def render(self, context):
        return (self.nodelist_true.render(context) + '\n' +
                self.nodelist_false.render(context))


class VariableNodeAlertingOnUnescapeUse(debug.DebugVariableNode):
    def __init__(self, filter_expression, callback_func=None):
        super(VariableNodeAlertingOnUnescapeUse, self).__init__(
            filter_expression)
        self.__callback_func = callback_func

    def render(self, context):
        msg = None
        escaped = False
        if node_has_a_filter(self, 'force_escape'):
            escaped = True
        if not context.autoescape:
            if node_has_a_filter(self, 'escape_filter') and (
                    not node_has_a_filter(self, 'safe')):
                escaped = True
            msg = "In a context where autoescaping has been disabled."
        elif node_has_a_filter(self, 'safe'):
            msg = "Has the 'safe' template filter and will not be escaped."
        if not escaped and msg and self.__callback_func:
            source_node = getattr(context, 'source_node', None)
            result = UnEscapedVariableFinding(
                self, msg=msg, source_node=source_node)
            self.__callback_func(result)
        try:
            return super(VariableNodeAlertingOnUnescapeUse, self).render(
                context)
        except TypeError:
            return 'An error occured in rendering : %s %s' % (
                repr(self),
                'due to the workarounds for custom template filters')


class DebugNodeListOverLoad(debug.DebugNodeList):
    """ this is used to overload the DebugNodeList class so that
        extra context information can be provided to find the 'source node'
        of an un-escaped variable.
        e.g. the context of 'BaseIncludeNode' instances have a
        'source_node' attribute
    """

    def render_node(self, node, context):
        source_node_name = 'source_node'
        _include_node_name = 'IncludeNode'
        _prev_include_node_name = 'BaseIncludeNode'
        if hasattr(django.template.loader_tags,
                   _prev_include_node_name):
            _include_node_name = _prev_include_node_name
        if isinstance(node, getattr(django.template.loader_tags,
                                    _include_node_name)):
            new_context = copy.copy(context)
            if not hasattr(new_context, source_node_name):
                setattr(new_context, source_node_name, node)
            context = new_context
        return super(DebugNodeListOverLoad, self).render_node(
            node, context)


class ParserThatIdentifiesUnescapedVariable(debug.DebugParser):
    def _set_var_callback_func(self, var_callback_func):
        self.var_callback_func = var_callback_func

    def create_nodelist(self):
        return DebugNodeListOverLoad()

    def create_variable_node(self, filter_expression):
        var_callback_func = getattr(self, 'var_callback_func', None)
        return VariableNodeAlertingOnUnescapeUse(
            filter_expression, var_callback_func)

    def find_filter(self, filter_name):
        if filter_name not in self.filters:
            return lambda s, *args: filter_name
        return super(ParserThatIdentifiesUnescapedVariable, self).find_filter(
            filter_name)

    def compile_filter(self, token):
        try:
            return FilterExpressionExtraVarResolution(token, self)
        except django.template.base.TemplateSyntaxError:
            return FilterExpressionIgnoresArgCheck(token, self)


def _add_missing_to_context(var, context):
    """ adds the variable and its lookup part to the context.
        returns the altered context.
    """
    current = context
    if var.lookups is None:
        return context
    for bit in var.lookups:
        if bit in current and isinstance(current, InceptionDictionary):
            current = current[bit]
        else:
            value = "<var var='%s' hash='%s' findme: %s />" % (
                var, hash(var),
                "injectedattributevalue='%s'" % var)
            current[bit] = InceptionDictionary({'__self__': value})
            current = current[bit]
    return context


class FilterExpressionExtraVarResolution(
        django.template.base.FilterExpression):
    def resolve(self, context, ignore_failures=False):
        """ because django's FilterExpression swallows VariableDoesNotExist
            exceptions and we want to ensure that missing variables have some
            content.
        """
        if not isinstance(self.var, django.template.base.Variable):
            context = self._fill_in_missing_variable_filter_args(
                context)
            return super(FilterExpressionExtraVarResolution, self).resolve(
                context, ignore_failures)
        else:
            try:
                obj = self.var.resolve(context)
            except django.template.base.VariableDoesNotExist:
                pass
            """ fix up the context """
            _add_missing_to_context(self.var, context)
            context = self._fill_in_missing_variable_filter_args(context)
            return super(FilterExpressionExtraVarResolution, self).resolve(
                context, ignore_failures)

    def _fill_in_missing_variable_filter_args(self, context):
        """ returns context filled in with missing variable
            that were provided as an arg.
        """
        for func, args in self.filters:
            for lookup, arg in args:
                if lookup:
                    try:
                        arg.resolve(context)
                    except django.template.base.VariableDoesNotExist:
                        _add_missing_to_context(arg, context)
        return context


class FilterExpressionIgnoresArgCheck(FilterExpressionExtraVarResolution):
    def args_check(name, func, provided):
        """ always return true """
        return True

    args_check = staticmethod(args_check)


def compile_string(template_string, origin, callback=None):
    """ Compiles template_string into NodeList ready for rendering
        using the custom Parser.
    """
    lexer = debug.DebugLexer(template_string, origin)
    filtered_tokens = []
    default_tags = set(debug.DebugParser([]).tags.keys())
    to_add = set()
    for tag in default_tags:
        to_add.add('end' + tag)
    default_tags = default_tags.union(to_add)
    default_tags = default_tags.union({"else", "elif"})
    for token in lexer.tokenize():
        skip = False
        if token.token_type == 2:  # TOKEN_BLOCK
            splitted = token.contents.split()
            if splitted:
                command = splitted[0]
                if command in {"load", "url"}:
                    skip = True
                if command not in default_tags:
                    skip = True
        if not skip:
            filtered_tokens.append(token)

    parser = ParserThatIdentifiesUnescapedVariable(filtered_tokens)
    parser._set_var_callback_func(callback)
    return parser.parse()


class UnEscapedFinding(object):
    """ this is the base class for representing an un-escaped finding """

    def __init__(self, **kwargs):
        pass

    def get_line_number(self):
        raise NotImplementedError("not implemented!")

    def get_vulnerability_text(self):
        raise NotImplementedError("not implemented!")

    def get_filename(self):
        raise NotImplementedError("not implemented!")

    def __str__(self):
        return "%s %s %s" % (self.get_line_number(),
                             self.get_vulnerability_text(),
                             self.get_filename())

    def _get_reason(self):
        """ returns a reason behind the finding
            e.g. a variable has been marked safe.
        """
        raise NotImplementedError("not implemented!")

    def __unicode__(self):
        return six.text_type(self.__str__())


class UnEscapedVariableFinding(UnEscapedFinding):
    """ this class represents an un-escaped variable finding. """

    def __init__(self, var_node, **kwargs):
        super(UnEscapedVariableFinding, self).__init__(**kwargs)
        self._var_node = var_node
        self.source_node = kwargs.get('source_node', None)
        self.msg = kwargs.get('msg', '')

    def _get_var_node_source_info(self):
        part = self.get_vulnerability_text()
        var_node = self._var_node
        if self.source_node:
            var_node = self.source_node
        temp_v_info = _get_var_node_source_info(var_node)
        line_number = temp_v_info[0]
        filename = temp_v_info[2]
        return line_number, part, filename

    def get_line_number(self):
        return self._get_var_node_source_info()[0]

    def get_vulnerability_text(self):
        """ returns the vulnerability text for this finding. """
        return _get_var_node_source_info(self._var_node)[1]

    def get_filename(self):
        return self._get_var_node_source_info()[2]

    def _get_reason(self):
        return self.msg

    def __str__(self):
        super_str = super(UnEscapedVariableFinding, self).__str__()
        s_node_info = None
        if self.source_node:
            s_node_info = _get_var_node_source_info(self.source_node)[:-1]
        return "%s %s (source_node %s) \n\t\t%s" % (self._var_node,
                                                    self.msg, s_node_info,
                                                    super_str)


class BaseVariableContextFinding(UnEscapedFinding):
    """ this class is an extension of UnEscapedFinding for context variable
        finding results.
    """

    def __init__(self, var_node, **kwargs):
        super(BaseVariableContextFinding, self).__init__(**kwargs)
        self._var_node = var_node
        self._line_number = kwargs.get('line_number')
        self._filename = six.text_type(kwargs.get('filename'))
        self._vulnerability_text = six.text_type(
            kwargs.get('vulnerability_text'))

    def get_line_number(self):
        return self._line_number

    def get_vulnerability_text(self):
        return self._vulnerability_text

    def get_filename(self):
        return self._filename

    def __str__(self):
        super_str = super(BaseVariableContextFinding, self).__str__()
        return "%s %s \n\t\t%s" % (self._var_node, self._get_reason(),
                                   super_str)


class UnEscapedVarJavascriptContextFinding(BaseVariableContextFinding):
    """ this class represents an un-escaped variable finding
        in a javascript context.
    """

    def _get_reason(self):
        return "In a javascript context without the escapejs filter."


class UnQuotedVarElementAttributeContext(BaseVariableContextFinding):
    """ this class represents a variable finding
        in a html element attribute context which is not quoted.
    """

    def _get_reason(self):
        return "In a html element attribute context without being quoted."


class CompileStringWrapper(object):
    """ a class that wraps calling compile_string so as to provide a
        'callback' func
    """

    def __init__(self, callback_func=None, store_results=True):
        self.store_results = store_results
        self.callback_func = callback_func
        self.results = []

    def compile_string(self, template_string, origin):
        return compile_string(template_string, origin, self.handle_callback)

    def handle_callback(self, result, **kwargs):
        if self.store_results:
            self.add_result(result)
        if self.callback_func is not None:
            self.callback_func(result, **kwargs)

    def add_result(self, result):
        self.results.append(result)


def get_default_context():
    return Context({'csrf_token': 'csrf_token'})


def get_template_source(template_name):
    """ returns the source and 'origin' for a template name. """
    for loader in django.template.loader.template_source_loaders:
        try:
            source, origin = loader.load_template_source(
                template_name)
            if source:
                return source, origin
        except django.template.base.TemplateDoesNotExist:
            pass
    return None, None


def __get_template_source_info_from_kwargs(template, **kwargs):
    source = kwargs.get('source', None)
    origin_fname = kwargs.get('origin_fname', None)
    if source is None or origin_fname is None:
        source, origin_fname = get_template_source(template.name)
    return source, origin_fname


def get_non_js_escaped_results_for_template(template, **kwargs):
    """ returns a generator of UnEscapedJavascriptContextFinding results for
        a given template
    """
    source, origin_fname = __get_template_source_info_from_kwargs(
        template, **kwargs)
    if not source or source is None:
        raise ValueError("source is empty")
    try:
        doc = lxml.html.fromstring(source)
    except lxml.etree.ParserError:
        raise ValueError("could not parse source")
    for block in doc.xpath(".//script"):
        text = block.text_content()
        origin = django.template.base.StringOrigin(text)
        for node in compile_string(text, origin).get_nodes_by_type(
                debug.DebugVariableNode):
            if node_has_a_filter(node, 'escapejs_filter'):
                continue
            string_range = node.source[1]
            line_no = (block.sourceline +
                       text[: string_range[0]].count("\n"))
            part = text[string_range[0]: string_range[1]]
            result = UnEscapedVarJavascriptContextFinding(
                var_node=node, line_number=line_no,
                filename=origin_fname, vulnerability_text=part)
            yield result


def get_non_quoted_attr_vars_for_template(template, **kwargs):
    """ returns a generator of UnQuotedVarElementAttributeContext results
        for a given template.
    """
    from . import util

    source, origin_fname = __get_template_source_info_from_kwargs(
        template, **kwargs)
    if not source or source is None:
        raise ValueError("source is empty")
    orig = source
    for line_no, content in enumerate(orig.split("\n"), 1):
        while True:
            _end = -1
            _start = content.find("<")
            if _start != -1:
                _end = content.find(">")
            if _start == -1 or _end == -1:
                break
            if content[_end] != ">":
                raise ValueError("Invalid end tag index %s %s" % (
                    content[_end], _end))
            current = content[_start: _end]
            content = content[_end + 1:]
            non_q_text = util.get_non_quoted_content(current)
            origin = django.template.base.StringOrigin(non_q_text)
            if django.template.base.VARIABLE_TAG_START not in non_q_text:
                continue
            try:
                nodelist = compile_string(non_q_text, origin)
            except django.template.base.TemplateSyntaxError:
                nodelist = None
            if not nodelist:
                continue
            for node in nodelist.get_nodes_by_type(
                    debug.DebugVariableNode):
                part = non_q_text[node.source[1][0]:node.source[1][1]]
                res = UnQuotedVarElementAttributeContext(
                    var_node=node, line_number=line_no,
                    filename=origin_fname,
                    vulnerability_text=part)
                yield res



from django.template.loader import BaseLoader, get_template_from_string


class Loader(BaseLoader):
    """ This loader will always return an 'empty' template and *therefore*
        *should* be the last loader in settings.TEMPLATE_LOADERS.
    """
    is_usable = True

    def load_template(self, template_name, template_dirs=None):
        template = get_template_from_string('', None, template_name)
        return template, None

    def load_template_source(self, template_name, template_dirs=None):
        return '', None



import django
import os
import warnings

from django.conf import settings
from django.template import loader
from django.utils.encoding import smart_text

from . import parse_template


def configure_django(template_dirs):
    """ this configures django by calling settings.configure.
        note: settings.configure can only be called once!
    """
    TEMPLATE_LOADERS = (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
        'django_xss_detection.loaders.nop.Loader',
    )
    settings.configure(DEBUG=False, TEMPLATE_DEBUG=True,
                       TEMPLATE_DIRS=template_dirs,
                       TEMPLATE_LOADERS=TEMPLATE_LOADERS)
    if hasattr(django, 'setup'):
        django.setup()


def patch(csw):
    django.template.base.compile_string = csw.compile_string
    django.template.defaulttags.IfChangedNode = parse_template.\
        IfChangedNodeOverload
    django.template.defaulttags.IfNode = parse_template.IfNodeOverload
    django.template.defaulttags.IfEqualNode = parse_template.\
        IfEqualNodeOverload
    django.template.base.add_to_builtins(
        'django_xss_detection.templatetags.waffle')


def get_template_wrapped(template_name):
    """ returns the result of calling loader.get_template(template_name)
        if the template can be loaded.
        otherwise returns None.
    """
    try:
        return loader.get_template(template_name)
    except (django.template.base.TemplateSyntaxError, UnicodeDecodeError) as e:
        msg = "skipping %s %s" % (template_name, repr(e))
        warnings.warn(msg)
    return None


def is_vuln_in_parent(result, template_name):
    """ returns True if the vulnerability is found within the parent of
        the given UnEscapedVariableFinding.
    """
    f_name = result.get_filename()
    if f_name and not f_name.endswith(template_name):
        """ ^ found in parent. """
        return True
    return False


def is_vuln_already_in_include(result, input_results):
    """ returns true if the given result is already present in the
        'included' file.
    """
    if not hasattr(result, 'source_node'):
        return False
    if not (result.source_node and result._var_node.source):
        return False
    lookup_name = result._var_node.source[0].loadname
    if lookup_name not in input_results:
        return False
    result_v_info = parse_template._get_var_node_source_info(result._var_node)
    for _res in input_results[lookup_name]:
        _rv_info = parse_template._get_var_node_source_info(
            _res._var_node)
        if _rv_info == result_v_info:
            return True
    return False


def uniquify_results(input_results):
    """ transforms a set of results per a file into a set of
        unique results - holding for different files.

        This includes:
        removing results that are found in the parent
        of a given template,

        removing results that are found in a template
        through being included but already exist in the
        included template.
    """
    ret = {}
    for template_name, results in input_results.items():
        new_results = []
        for result in results:
            skip = False
            if is_vuln_in_parent(result, template_name):
                skip = True
            elif is_vuln_already_in_include(result, input_results):
                skip = True
            if not skip:
                new_results.append(result)
        if new_results:
            ret[template_name] = new_results
    return ret


def walk_templates(template_dirs):
    templates = (os.path.relpath(os.path.join(root, _file), template_dir)
                 for template_dir in template_dirs
                 for root, dirs, files in os.walk(smart_text(template_dir))
                 for _file in files)
    results = {}
    for templ in templates:
        csw = parse_template.CompileStringWrapper()
        patch(csw)
        template = get_template_wrapped(templ)
        context = parse_template.get_default_context()
        if template is None:
            continue
        _source, _origin_fname = parse_template.get_template_source(templ)
        for method in [parse_template.get_non_js_escaped_results_for_template,
                       parse_template.get_non_quoted_attr_vars_for_template]:
            try:
                for result in method(template, source=_source,
                                     origin_fname=_origin_fname):
                    csw.handle_callback(result)
            except ValueError as e:
                warnings.warn("could not call %s, %s" % (
                    method.__name__, e))
        try:
            template.render(context)
        except (django.template.base.TemplateSyntaxError,
                django.template.base.TemplateDoesNotExist,
                TypeError) as e:
            msg = "skipping %s %s" % (templ, repr(e))
            warnings.warn(msg)
        if csw.results:
            results[templ] = csw.results
    return uniquify_results(results)


def get_non_quoted_content(content):
    ret = []
    quote_chrs = {"'", '"'}
    if not quote_chrs.intersection(set(content)):
        return content
    inside_quote = None
    for ch in content:
        if ch in quote_chrs:
            if inside_quote is None:
                inside_quote = ch
            elif ch == inside_quote:
                inside_quote = None
        else:
            if inside_quote is None:
                ret.append(ch)
    return ''.join(ret)



from __future__ import print_function
import argparse
import collections
import json
import logging

from . import util


def setup_option():
    opt = argparse.ArgumentParser(
        description="Find potential xss bugs in a django app!")
    opt.add_argument(
        "-d", "--template-directory", dest="template_dirs",
        action="append", help="Specify a template directory. "
        "This argument can be specified multiple times.", required=True)
    opt.add_argument(
        "-j", "--json", dest="json_output",
        action="store_true", help="Print results out as JSON.")
    return opt


def main(template_dirs, json_output=False, **kwargs):
    util.configure_django(template_dirs)
    if 'logging_capture_warnings' in kwargs:
        logging.captureWarnings(kwargs.get('logging_capture_warnings'))
    f_results = util.walk_templates(template_dirs)
    if json_output:
        _output_results_in_json(f_results)
    else:
        for template_name, results in f_results.items():
            print(template_name)
            for result in results:
                print('    ', result)


def _output_results_in_json(f_results):
    """ Prints out results in JSON in the following format:
        {'filename' : [{'result ...}, 'filename_two' : [...] }
    """
    out = collections.defaultdict(list)
    for filename, results in f_results.items():
        for result in results:
            result_dict = {
                'line_number': result.get_line_number(),
                'finding_reason': result._get_reason(),
                'vulnerability_text': result.get_vulnerability_text(),
            }
            out[result.get_filename()].append(result_dict)
    print(json.dumps(out))


def from_cli():
    opt = setup_option()
    args = opt.parse_args()
    main(args.template_dirs, args.json_output, logging_capture_warnings=False)


if __name__ == "__main__":
    from_cli()



#!/usr/bin/env python
from setuptools import setup, find_packages
import os


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='django-xss-detection',
    author='David Black',
    author_email='dblack@atlassian.com',
    url='https://bitbucket.org/atlassian/django_xss_detection',
    packages=find_packages(),
    description=read('README.md'),
    long_description=read('README.md'),
    version=__import__('django_xss_detection').__version__,
    test_suite='django_xss_detection.test',
    install_requires=[
        'Django>=1.5',
        'lxml',
    ],
    platforms=['any'],
    license='BSD',
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'License :: OSI Approved :: BSD License',
        'Framework :: Django',
    ],
)



from django import template
from . import SimpleConditionNode
register = template.Library()


@register.tag('flag')
def flag(parser, token):
    return SimpleConditionNode.handle_token(parser, token, 'flag')


@register.tag('switch')
def switch(parser, token):
    return SimpleConditionNode.handle_token(parser, token, 'switch')


@register.tag('sample')
def sample(parser, token):
    return SimpleConditionNode.handle_token(parser, token, 'sample')
